#include <IRremote.h>
#include <Adafruit_GFX.h> // for OLED
#include <Adafruit_SSD1306.h> // for OLED
//#include <EEPROM.h>

const int IR_PIN = 2;
IRrecv irDetect(IR_PIN);
decode_results irIn;
/*--------------for OLED---------------*/
#define OLED_RESET 4
Adafruit_SSD1306 display(OLED_RESET);
/*-------------------------------------*/

byte key = 0;
int player_score = 0;
byte player_score_right = 0;
byte player_score_wrong = 0;
byte first_num = 0;
byte second_num = 0;
byte operation = 0;
String example = "";
byte result = 0;
String key_str = "";
String acc = "";
byte time_left = 60;
unsigned long game_length = 60000;
boolean level = false; // false -> "+"; true -> "*"

void setup() {
  irDetect.enableIRIn(); // Start the Receiver
  init_OLED();
  hello_msg();
}

void loop() {
  randomSeed(analogRead(0));
  choose_level();
  while (true) {
    unsigned long previousMillis = millis();
    while (millis() - previousMillis <= game_length) {
      game();
    }
    game_res_player1();
    player_score = 0;
    player_score_right = 0;
    player_score_wrong = 0;
    break;
  }
}
/*------------------------------------------------*/
void IR() {
  if (irDetect.decode(&irIn)) {
    key = decodeIR();
    irDetect.resume();
  }
}
/*------------------------------------------------*/
byte decodeIR() { // Indicate what key is pressed
  byte k = 0;
  switch (irIn.value)
  {
    case 0xFF02FD:
      //  Serial.println("OK");
      k = 10;
      break;

    case 0xFF6897:
      //   Serial.println("1");
      key_str = "1";
      break;

    case 0xFF9867:
      //  Serial.println("2");
      key_str = "2";
      break;

    case 0xFFB04F:
      //  Serial.println("3");
      key_str = "3";
      break;

    case 0xFF30CF:
      //   Serial.println("4");
      key_str = "4";
      break;

    case 0xFF18E7:
      //   Serial.println("5");
      key_str = "5";
      break;

    case 0xFF7A85:
      //  Serial.println("6");
      key_str = "6";
      break;

    case 0xFF10EF:
      //   Serial.println("7");
      key_str = "7";
      break;

    case 0xFF38C7:
      //   Serial.println("8");
      key_str = "8";
      break;

    case 0xFF5AA5:
      //   Serial.println("9");
      key_str = "9";
      break;

    case 0xFF4AB5:
      //   Serial.println("0");
      key_str = "0";
      break;

    case 0xFF42BD:
      //    Serial.println("*");
      k = 11;
      break;

    case 0xFF52AD:
      //   Serial.println("#");
      k = 12;
      break;

    default:
      break;
  }
  return k;
}
/*------------------------------------------------*/
void init_OLED() {
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);  // initialize with the I2C addr 0x3C (for the 128x32)
  display.clearDisplay(); // clear display
  display.ssd1306_command(SSD1306_SETCONTRAST);
  display.ssd1306_command(0); // Where c is a value from 0 to 255 (sets contrast e.g. brightness)
  display.setTextSize(1); // set text size
  display.setTextColor(WHITE); // set text color
}
/*------------------------------------------------*/
void hello_msg() {
  display.setCursor(48, 0);
  display.print("GAME");
  display.setCursor(0, 8);
  display.setTextSize(2); // set text size
  display.print("ARITHMETIC");
  display.setTextSize(1); // set text size
  display.setCursor(22, 24);
  display.print("Robohobby.by");
  display.display();
  delay(3000);
  display.clearDisplay(); // clear display
}
/*------------------------------------------------*/
void choose_level() {
  display.setCursor(0, 0);
  display.print("Choose game level");
  display.setCursor(0, 8);
  display.setTextSize(1.5);
  display.print("# - simple [ + ]");
  display.setCursor(0, 16);
  display.print("* - advanced [ * ]");
  display.display();

  while (true) {
    IR();
    if (key == 11) {
      level = true;
      break;
    }
    if (key == 12) {
      level = false;
      break;
    }
  }
  display.clearDisplay(); // clear display

}
/*------------------------------------------------*/
void gen (boolean mode) {
  example = "";
  first_num = random(2, 10);
  second_num = random(2, 10);
  operation = random(0, 2);
  if (mode) {
    result = first_num * second_num;
    if (operation) {
      example = String (first_num);
      example += " x ";
      example += String (second_num);
      example += " = ";
    }
    else {
      example = String (result);
      example += " / ";
      example += String (second_num);
      example += " = ";
      result = first_num;
    }
  }
  else {
    result = first_num + second_num;
    if (operation) {
      example = String (first_num);
      example += " + ";
      example += String (second_num);
      example += " = ";
    }
    else {
      example = String (result);
      example += " - ";
      example += String (second_num);
      example += " = ";
      result = first_num;
    }
  }
}
/*------------------------------------------------*/

void game() {

  display.clearDisplay(); // clear display
  display.setTextSize(1); // set text size
  display.setCursor(0, 0);
  display.print("PLAYER SCORE ");
  display.print(player_score);

  display.setCursor(0, 24);
  display.print("Time left ");
  display.print(time_left);
  display.print(" s");
  display.setTextSize(2); // set text size
  display.setCursor(0, 8);
  gen(level);
  display.print(example);
  display.display();

  while (true) {
    static unsigned long lastTick = 0;
    if (millis() - lastTick >= 1000) {
      lastTick = millis();
      time_left--;
    }

    key = 0;
    key_str = "";
    IR();
    acc += key_str;
    display.print(key_str);
    display.display();

    if (key == 10 && acc != "") {
      if (acc.toInt() == result) {
        player_score_right++;
      } 
      else {
        player_score_wrong++;
      }
      player_score = player_score_right - player_score_wrong;
      acc = "";
      break;

    }
  }

}
/*------------------------------------------------*/
void game_res_player1 () {
  display.clearDisplay(); // clear display
  display.setCursor(8, 8);
  display.setTextSize(2); // set text size
  display.print("GAME OVER");
  display.display();
  delay(2000);
  display.clearDisplay(); // clear display

  display.setCursor(16, 0);
  display.setTextSize(1); // set text size
  if (level) {
    display.print("*");
  }
  else {
    display.print("+");
  }
  display.setCursor(25, 0);
  display.print(" PLAYER SCORE");
  example = String (player_score);
  example += " (";
  example += String (player_score_right);
  example += "-";
  example += String (player_score_wrong);
  example += ")";
  display.setCursor(16, 10);
  display.setTextSize(2); // set text size
  display.print(example);
  display.display();

  while (true) {
    key = 0;
    IR();
    if (key == 10) break;
  }

  key = 0;
  acc = "";
  player_score = 0;
  player_score_right = 0;
  player_score_wrong = 0;
  time_left = 60;

  display.clearDisplay(); // clear display
  display.setTextSize(1); // set text size
  display.display();
}
/*------------------------------------------------*/
